#include "buffer.h"
#include "strerr.h"
#include "stralloc.h"
#include "alloc.h"
#include "env.h"
#include "cdb.h"

#define FATAL "tcprulescheck: fatal: "

char *fnrules;
int fdrules;
static struct cdb c;

void die_usage(void)
{
  strerr_die1x(100,"tcprulescheck: usage: tcprulescheck rules.cdb");
}
void die_read(void)
{
  strerr_die4sys(111,FATAL,"unable to read ",fnrules,": ");
}
void nomem(void)
{
  strerr_die2x(111,FATAL,"out of memory");
}

static stralloc tmp = {0};

void dorule(void)
{
  char *data;
  unsigned int datalen;
  unsigned int next0;

  switch(cdb_find(&c,tmp.s,tmp.len)) {
    case -1: die_read();
    case 0: return;
  }

  datalen = cdb_datalen(&c);
  data = alloc(datalen);
  if (!data) nomem();
  if (cdb_read(&c,data,datalen,cdb_datapos(&c)) != 0) die_read();

  buffer_puts(buffer_1,"rule ");
  buffer_put(buffer_1,tmp.s,tmp.len);
  buffer_puts(buffer_1,":\n");
  while ((next0 = byte_chr(data,datalen,0)) < datalen) {
    switch(data[0]) {
      case 'D':
	buffer_puts(buffer_1,"deny connection\n");
	buffer_flush(buffer_1);
	_exit(0);
      case '+':
	buffer_puts(buffer_1,"set environment variable ");
	buffer_puts(buffer_1,data + 1);
	buffer_puts(buffer_1,"\n");
	break;
    }
    ++next0;
    data += next0; datalen -= next0;
  }
  buffer_puts(buffer_1,"allow connection\n");
  buffer_flush(buffer_1);
  _exit(0);
}

main(int argc,char **argv)
{
  char *tcpremoteip;
  char *tcpremoteinfo;
  char *remotehost;

  fnrules = argv[1];
  if (!fnrules) die_usage();

  tcpremoteip = env_get("TCPREMOTEIP");
  if (!tcpremoteip) tcpremoteip = "0.0.0.0";
  tcpremoteinfo = env_get("TCPREMOTEINFO");
  remotehost = env_get("TCPREMOTEHOST");

  fdrules = open_read(fnrules);
  if (fdrules == -1) die_read();
  cdb_init(&c,fdrules);

  if (tcpremoteinfo) {
    if (!stralloc_copys(&tmp,tcpremoteinfo)) nomem();
    if (!stralloc_cats(&tmp,"@")) nomem();
    if (!stralloc_cats(&tmp,tcpremoteip)) nomem();
    dorule();

    if (remotehost) {
      if (!stralloc_copys(&tmp,tcpremoteinfo)) nomem();
      if (!stralloc_cats(&tmp,"@=")) nomem();
      if (!stralloc_cats(&tmp,remotehost)) nomem();
      dorule();
    }
  }

  if (!stralloc_copys(&tmp,tcpremoteip)) nomem();
  dorule();

  if (remotehost) {
    if (!stralloc_copys(&tmp,"=")) nomem();
    if (!stralloc_cats(&tmp,remotehost)) nomem();
    dorule();
  }

  if (!stralloc_copys(&tmp,tcpremoteip)) nomem();
  while (tmp.len > 0) {
    if (tcpremoteip[tmp.len - 1] == '.')
      dorule();
    --tmp.len;
  }

  if (remotehost) {
    while (*remotehost) {
      if (*remotehost == '.') {
        if (!stralloc_copys(&tmp,"=")) nomem();
        if (!stralloc_cats(&tmp,remotehost)) nomem();
	dorule();
      }
      ++remotehost;
    }
  }

  dorule();

  buffer_putsflush(buffer_1,"default:\nallow connection\n");
  _exit(0);
}
